/*
 * cyberphotos
 * Thomas Nemeth, le 15.05.2004
 *
 *   Copyright (C) 1999  Thomas Nemeth
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <locale.h>
#include <string.h>
#include <strings.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <errno.h>
#include <dirent.h>
#include <ctype.h>
#include <time.h>
#include <fcntl.h>
#include <getopt.h>
#include "defines.h"
#include "xmem.h"
#include "xwindow.h"
#include "xfunctions.h"
#include "tools.h"
#include "cmdstrs.h"


#define IS_FIELD(line, field) strncmp(line, field, strlen(field)) == 0

#define USBDEVICES "/proc/bus/usb/devices"

#define END 0
#define USB 1
#define CFG 2

int end_program = FALSE;
int verbose_lvl = NORMAL;
int simulate    = FALSE;


static char   *all_locale;
/* app's configuration */
static char   *config_file  = NULL;
static int     is_plugged   = FALSE;
static int     watch_config = FALSE;
static int     use_date     = TRUE;
static int     x11          = FALSE;
/* Images access */
static char   *mountpoint   = NULL;
static char   *product      = NULL;
static char   *srcdir       = NULL;
static char   *dstdir       = NULL;
static int     move_files   = FALSE;



void usage(int help) {
    if (help) {
        printv(FORCE,
               "Usage : " PROGRAM " [-hVvqwxs] [-f file]\n"
               " -h            : this help\n"
               " -V            : program version\n"
               " -q            : quiet mode\n"
               " -v            : verbose mode\n"
               " -w            : watch configuration file's changes\n"
               " -x            : use an X11 alert window\n"
               " -s            : simulate (don't really move files)\n"
               " -f file       : configuration file\n"
               "\nConfiguration file entries (separator is `=') :\n"
               "  Verbose    : Verbose mode (QUIET/NORMAL/VERBOSE)\n"
               "  MountPoint : mount point for product (string)\n"
               "  Product    : product's product ID (string)\n"
               "  SrcDir     : product's source directory (string)\n"
               "  DstDir     : local destination directory (string)\n"
               "  Move       : move files, instead of copy (boolean)\n"
               "  Watch      : watch configuration file's changes (boolean)\n"
               "  UseDate    : use date for storing photographs (boolean)\n"
               "  X11        : use an X11 alert window (boolean)\n"
               "  Splash     : display splash screen (seconds, 0=no)\n"
               "  Geometry   : window's geometry (geometry string)\n"
               "  Font       : font (font string)\n"
               "  FGColor    : color for X11 texts (color string)\n"
               "  BGColor    : color for X11 background (color string)\n"
               "\n");
    } else {
        printv(FORCE, PACKAGE " v" VERSION
               " (" OSTYPE "-" MACHTYPE "),  Thomas Nemeth 2004.\n");
    }
}


void parse_argv(int argc, char *argv[]) {
    int   option;

    while ((option = getopt(argc, argv, "hVvqwxsf:")) != -1) {
        switch (option) {
            case 'h' :
                usage(1);
                exit(0);
                break;
            case 'V' :
                usage(0);
                exit(0);
                break;
            case 'v' :
                verbose_lvl = VERBOSE;
                break;
            case 'q' :
                verbose_lvl = QUIET;
                break;
            case 'w' :
                watch_config = !watch_config;
                break;
            case 'x' :
                x11 = !x11;
                break;
            case 's' :
                simulate = !simulate;
                break;
            case 'f' :
                config_file = xstrdup(optarg);
                break;
            default :
                printv(ERROR, "Unkown option : %c\n", option);
                usage(1);
                exit(-1);
                break;
        }
    }
}


int getbool(char *value) {
    int i;
    for (i = 0 ; value[i] ; i++) value[i] = tolower(value[i]);
    if (strcmp(value, "0")     == 0) return FALSE;
    if (strcmp(value, "1")     == 0) return TRUE;
    if (strcmp(value, "true")  == 0) return TRUE;
    if (strcmp(value, "false") == 0) return FALSE;
    if (strcmp(value, "yes")   == 0) return TRUE;
    if (strcmp(value, "no")    == 0) return FALSE;
    if (strcmp(value, "on")    == 0) return TRUE;
    if (strcmp(value, "off")   == 0) return FALSE;
    printv(ERROR, "Error in converting \"%s\" to boolean value.\n", value);
    return FALSE;
}


void load_config() {
    FILE *file;
    int  i = 0;

    if ((file = fopen(config_file, "r")) == NULL) return;
    while (! feof(file)) {
        char line[MAXSTRLEN + 1];
        char *value;

        bzero(line, MAXSTRLEN + 1);
        fgets(line, MAXSTRLEN, file);
        i++;
        if (line[strlen(line) - 1] == '\n') line[strlen(line) - 1] = 0;
        if ((line[0] == '#') || (line[0] == 0)) continue;

        value = strchr (line, '=') + 1;
        while ((value[0] == ' ') && (value[0] != 0)) value++;
        if (value[0] == 0) continue;

        if (IS_FIELD(line, "Verbose")) {
            if (IS_FIELD(value, "QUIET"))
                verbose_lvl = QUIET;
            else if (IS_FIELD(value, "NORMAL"))
                verbose_lvl = NORMAL;
            else if (IS_FIELD(value, "VERBOSE"))
                verbose_lvl = VERBOSE;
            else
                verbose_lvl = NORMAL;
        } else if (IS_FIELD(line, "MountPoint")) {
            CHANGE_STRING(mountpoint, value);
        } else if (IS_FIELD(line, "Product")) {
            CHANGE_STRING(product, value);
        } else if (IS_FIELD(line, "SrcDir")) {
            CHANGE_STRING(srcdir, value);
        } else if (IS_FIELD(line, "DstDir")) {
            CHANGE_STRING(dstdir, value);
        } else if (IS_FIELD(line, "Move")) {
            move_files = getbool(value);
        } else if (IS_FIELD(line, "Watch")) {
            watch_config = getbool(value);
        } else if (IS_FIELD(line, "UseDate")) {
            use_date = getbool(value);
        } else if (IS_FIELD(line, "X11")) {
            x11 = getbool(value);
        } else if (IS_FIELD(line, "Splash")) {
            splash = atoi(value);
        } else if (IS_FIELD(line, "Font")) {
            CHANGE_STRING(fontname, value);
        } else if (IS_FIELD(line, "Geometry")) {
            CHANGE_STRING(geom, value);
        } else if (IS_FIELD(line, "FGColor")) {
            CHANGE_STRING(fgcolor, value);
        } else if (IS_FIELD(line, "BGColor")) {
            CHANGE_STRING(bgcolor, value);
        } else {
            printv(ERROR, "Error in %s at line %d :\n[%s].\n",
                   config_file, i, line);
        }
    }
    fclose(file);
}


void display_infos() {
    printv(VERBOSE, "Mount point  : %s\n", mountpoint);
    printv(VERBOSE, "Product ID   : %s\n", product);
    printv(VERBOSE, "Source dir.  : %s\n", srcdir);
    printv(VERBOSE, "Dest. dir.   : %s\n", dstdir);
    printv(VERBOSE, "Move files   : %s\n", move_files ? "yes" : "no");
    printv(VERBOSE, "Simulate     : %s\n", simulate ? "yes" : "no");
    printv(VERBOSE, "Watch config : %s\n", watch_config ? "yes" : "no");
    printv(VERBOSE, "Use date     : %s\n", use_date ? "yes" : "no");
    printv(VERBOSE, "X11          : %s\n", x11 ? "yes" : "no");
    printv(VERBOSE, "Font         : %s\n", fontname ? fontname : "DEFAULT");
    printv(VERBOSE, "Geometry     : %s\n", geom ? geom : "AUTO.");
    printv(VERBOSE, "FGColor      : %s\n", fgcolor ? fgcolor : "White");
    printv(VERBOSE, "BGColor      : %s\n", bgcolor ? bgcolor : "Black");
}


void tmms_init() {
   signal(SIGTERM, signals_handler);
   signal(SIGQUIT, signals_handler);
   signal(SIGINT,  signals_handler);

   all_locale = setlocale(LC_ALL, "");

    if (config_file) load_config();
    if (
            (! mountpoint) ||
            (! product)    ||
            (! srcdir)     ||
            (! dstdir)
        ) {
        printv(ERROR, "Error : not enough variables set !\n");
        exit(1);
    }

    display_infos();

    if (x11) init_x11();
}


int plugged() {
    FILE *file;
    int   ok = FALSE;

    if ((file = fopen(USBDEVICES, "r")) == NULL) return ok;
    while (!feof(file)) {
        char line[MAXSTRLEN + 1];
        bzero(line, MAXSTRLEN + 1);
        fgets(line, MAXSTRLEN, file);
        if (strstr(line, product) != NULL) {
            ok = TRUE;
            break;
        }
    }
    fclose(file);

    return ok;
}


int wait_event() {
    static time_t usb_mtime = 0;
    static time_t cfg_mtime = 0;
    int got_usb = FALSE, got_cfg = FALSE;

    while (!got_usb && !got_cfg && !end_program) {
        sleep(1);
        if (watch_config) got_cfg = watch_file(config_file, &cfg_mtime, FALSE);
        if (! got_cfg) got_usb = watch_file(USBDEVICES, &usb_mtime, TRUE);
        if (got_usb) {
            got_usb = plugged();
            if (got_usb && !is_plugged) {
                is_plugged = TRUE;
            } else if (got_usb && is_plugged) {
                got_usb = FALSE;
            } else if (!got_usb && is_plugged) {
                is_plugged = FALSE;
            }
        }
    }
    if (got_usb) return USB;
    if (got_cfg) return CFG;
    return END;
}


void do_mount(int on) {
    char *cmd;
    char *mnt;
    char *dst = mountpoint;
    int   len;

    if (on)
        mnt = "mount";
    else
        mnt = "umount";

    len = strlen(mnt) + strlen(dst) + 2;
    cmd = xmalloc(len + 1);
    snprintf(cmd, len, "%s %s", mnt, dst);
    my_system(cmd);
    free(cmd);
}


void file_operation(int *n, int m, const char *dirname, const char *filename) {
    time_t ctime = file_is_ok(dirname, filename);

    if (ctime != 0) {
        char *cpmv = move_files ? "mv" : "cp";
        char *cmd = NULL;
        char *info_dstdir = NULL;
   
        (*n)++;
        if (use_date) {
            char        info_year[6];
            struct tm  *ft;
            ft  = localtime(&ctime);
            cmd = get_dated_cmd(cpmv, dirname, filename, ctime, dstdir);
            snprintf(info_year, 5, "%04d/", ft->tm_year + 1900);
            info_dstdir = strstr(cmd, info_year);
        } else {
            cmd = get_normal_cmd(cpmv, dirname, filename, dstdir);
            info_dstdir = strrchr(dstdir, '/');
            if (! info_dstdir) info_dstdir = dstdir;
        }
        printv(NORMAL, "    %s -> %s (%d/%d)\n", filename, info_dstdir, *n, m);
        if (x11) display_datas(filename, info_dstdir, *n, m);
        printv(VERBOSE,"    Commande : %s\n", cmd);
        if (! simulate)
            my_system(cmd);
        else
            sleep(1);
        free(cmd);
    }
}


void parse_all_files() {
    DIR           *dir;
    struct dirent *dir_ent;
    char          *dirname;
    int            len;

    len = strlen(mountpoint) + strlen(srcdir) + 2;
    dirname = xmalloc(len + 1);
    snprintf(dirname, len, "%s/%s", mountpoint, srcdir);
    if ((dir = opendir(dirname)) != NULL) {
        int nbfiles = 0, i = 0, cont = TRUE;
        nbfiles = get_nb_files(dir, dirname);
        if (nbfiles > 0) {
            if (x11) cont = display_window(nbfiles);
            if (cont) {
                printv(VERBOSE, "    Lecture de %s : ", dirname);
                printv(VERBOSE, "%d fichier%s...\n",
                       nbfiles, nbfiles > 1 ? "s" : "");
                while ((dir_ent = readdir(dir)) != NULL) {
                    file_operation(&i, nbfiles, dirname, dir_ent->d_name);
                }
            }
        }
        closedir(dir);
        if (x11) close_window();
    }
    free(dirname);
}    


int main(int argc, char *argv[]) {
    parse_argv(argc, argv);
    tmms_init();
    while (!end_program) {
        switch (wait_event()) {
            case USB :
                printv(NORMAL, "Branchement de l'appareil dtect !\n");
                do_mount(TRUE);
                printv(VERBOSE, "  * APN mont.\n");
                parse_all_files();
                do_mount(FALSE);
                printv(VERBOSE, "  * APN dmont.\n");
                printv(NORMAL,  "  Fait.\n");
                break;
            case CFG :
                printv(NORMAL, "Configuration change : rechargement...\n");
                if (x11) deinit_x11();
                load_config();
                display_infos();
                if (x11) init_x11();
                break;
        }
    }
    printv(VERBOSE, "Arrt demand, libration de la mmoire...\n");
    if (x11) deinit_x11();
    FREE(geom);
    FREE(mountpoint);
    FREE(product);
    FREE(srcdir);
    FREE(dstdir);

    return 0; 
}

